@@ -18,7 +18,7 @@
 */
 
 #ifndef NUTCLIENT_HPP_SEEN
-#define NUTCLIENT_HPP_SEEN
+#define NUTCLIENT_HPP_SEEN 1
 
 /* Begin of C++ nutclient library declaration */
 #ifdef __cplusplus
@@ -28,6 +28,13 @@
 #include <map>
 #include <set>
 #include <exception>
+#include <cstdint>
+#include <ctime>
+
+/* See include/common.h for details behind this */
+#ifndef NUT_UNUSED_VARIABLE
+#define NUT_UNUSED_VARIABLE(x) (void)(x)
+#endif
 
 namespace nut
 {
@@ -51,9 +58,11 @@ class NutException : public std::exception
 {
 public:
 	NutException(const std::string& msg):_msg(msg){}
-	virtual ~NutException() throw() {}
-	virtual const char * what() const throw() {return this->_msg.c_str();}
-	virtual std::string str() const throw() {return this->_msg;}
+	NutException(const NutException&) = default;
+	NutException& operator=(NutException& rhs) = default;
+	virtual ~NutException() noexcept override;
+	virtual const char * what() const noexcept override {return this->_msg.c_str();}
+	virtual std::string str() const noexcept {return this->_msg;}
 private:
 	std::string _msg;
 };
@@ -65,7 +74,9 @@ class SystemException : public NutException
 {
 public:
 	SystemException();
-	virtual ~SystemException() throw() {}
+	SystemException(const SystemException&) = default;
+	SystemException& operator=(SystemException& rhs) = default;
+	virtual ~SystemException() noexcept override;
 private:
 	static std::string err();
 };
@@ -78,7 +89,9 @@ class IOException : public NutException
 {
 public:
 	IOException(const std::string& msg):NutException(msg){}
-	virtual ~IOException() throw() {}
+	IOException(const IOException&) = default;
+	IOException& operator=(IOException& rhs) = default;
+	virtual ~IOException() noexcept override;
 };
 
 /**
@@ -88,7 +101,9 @@ class UnknownHostException : public IOException
 {
 public:
 	UnknownHostException():IOException("Unknown host"){}
-	virtual ~UnknownHostException() throw() {}
+	UnknownHostException(const UnknownHostException&) = default;
+	UnknownHostException& operator=(UnknownHostException& rhs) = default;
+	virtual ~UnknownHostException() noexcept override;
 };
 
 /**
@@ -98,7 +113,9 @@ class NotConnectedException : public IOException
 {
 public:
 	NotConnectedException():IOException("Not connected"){}
-	virtual ~NotConnectedException() throw() {}
+	NotConnectedException(const NotConnectedException&) = default;
+	NotConnectedException& operator=(NotConnectedException& rhs) = default;
+	virtual ~NotConnectedException() noexcept override;
 };
 
 /**
@@ -108,9 +125,30 @@ class TimeoutException : public IOException
 {
 public:
 	TimeoutException():IOException("Timeout"){}
-	virtual ~TimeoutException() throw() {}
+	TimeoutException(const TimeoutException&) = default;
+	TimeoutException& operator=(TimeoutException& rhs) = default;
+	virtual ~TimeoutException() noexcept override;
 };
 
+/**
+ * Cookie given when performing async action, used to redeem result at a later date.
+ */
+typedef std::string TrackingID;
+
+/**
+ * Result of an async action.
+ */
+typedef enum
+{
+	UNKNOWN,
+	PENDING,
+	SUCCESS,
+	INVALID_ARGUMENT,
+	FAILURE,
+} TrackingResult;
+
+typedef std::string Feature;
+
 /**
  * A nut client is the starting point to dialog to NUTD.
  * It can connect to an NUTD then retrieve its device list.
@@ -122,7 +160,7 @@ class Client
 	friend class Variable;
 	friend class Command;
 public:
-	~Client();
+	virtual ~Client();
 
 	/**
 	 * Intend to authenticate to a NUTD server.
@@ -132,13 +170,13 @@ public:
 	 * \todo Is his method is global to all connection protocol or is it specific to TCP ?
 	 * \note Actually, authentication fails only if already set, not if bad values are sent.
 	 */
-	virtual void authenticate(const std::string& user, const std::string& passwd)throw(NutException)=0;
+	virtual void authenticate(const std::string& user, const std::string& passwd) = 0;
 
 	/**
 	 * Disconnect from the NUTD server.
 	 * \todo Is his method is global to all connection protocol or is it specific to TCP ?
 	 */
-	virtual void logout()throw(NutException)=0;
+	virtual void logout() = 0;
 
 	/**
 	 * Device manipulations.
@@ -151,29 +189,29 @@ public:
 	 * \param name Name of the device.
 	 * \return The device.
 	 */
-	virtual Device getDevice(const std::string& name)throw(NutException);
+	virtual Device getDevice(const std::string& name);
 	/**
 	 * Retrieve the list of all devices supported by UPSD server.
 	 * \return The set of supported devices.
 	 */
-	virtual std::set<Device> getDevices()throw(NutException);
+	virtual std::set<Device> getDevices();
 	/**
 	 * Test if a device is supported by the NUTD server.
 	 * \param dev Device name.
 	 * \return true if supported, false otherwise.
 	 */
-	virtual bool hasDevice(const std::string& dev)throw(NutException);
+	virtual bool hasDevice(const std::string& dev);
 	/**
 	 * Retrieve names of devices supported by NUTD server.
 	 * \return The set of names of supported devices.
 	 */
-	virtual std::set<std::string> getDeviceNames()throw(NutException)=0;
+	virtual std::set<std::string> getDeviceNames() = 0;
 	/**
 	 * Retrieve the description of a device.
 	 * \param name Device name.
 	 * \return Device description.
 	 */
-	virtual std::string getDeviceDescription(const std::string& name)throw(NutException)=0;
+	virtual std::string getDeviceDescription(const std::string& name) = 0;
 	/** \} */
 
 	/**
@@ -186,54 +224,60 @@ public:
 	 * \param dev Device name
 	 * \return Variable names
 	 */
-	virtual std::set<std::string> getDeviceVariableNames(const std::string& dev)throw(NutException)=0;
+	virtual std::set<std::string> getDeviceVariableNames(const std::string& dev) = 0;
 	/**
 	 * Retrieve names of read/write variables supported by a device.
 	 * \param dev Device name
 	 * \return RW variable names
 	 */
-	virtual std::set<std::string> getDeviceRWVariableNames(const std::string& dev)throw(NutException)=0;
+	virtual std::set<std::string> getDeviceRWVariableNames(const std::string& dev) = 0;
 	/**
 	 * Test if a variable is supported by a device.
 	 * \param dev Device name
 	 * \param name Variable name
 	 * \return true if the variable is supported.
 	 */
-	virtual bool hasDeviceVariable(const std::string& dev, const std::string& name)throw(NutException);
+	virtual bool hasDeviceVariable(const std::string& dev, const std::string& name);
 	/**
 	 * Retrieve the description of a variable.
 	 * \param dev Device name
 	 * \param name Variable name
 	 * \return Variable description if provided.
 	 */
-	virtual std::string getDeviceVariableDescription(const std::string& dev, const std::string& name)throw(NutException)=0;
+	virtual std::string getDeviceVariableDescription(const std::string& dev, const std::string& name) = 0;
 	/**
 	 * Retrieve values of a variable.
 	 * \param dev Device name
 	 * \param name Variable name
 	 * \return Variable values (usually one) if available.
 	 */
-	virtual std::vector<std::string> getDeviceVariableValue(const std::string& dev, const std::string& name)throw(NutException)=0;
+	virtual std::vector<std::string> getDeviceVariableValue(const std::string& dev, const std::string& name) = 0;
 	/**
 	 * Retrieve values of all variables of a device.
 	 * \param dev Device name
 	 * \return Variable values indexed by variable names.
 	 */
-	virtual std::map<std::string,std::vector<std::string> > getDeviceVariableValues(const std::string& dev)throw(NutException);
+	virtual std::map<std::string,std::vector<std::string> > getDeviceVariableValues(const std::string& dev);
+	/**
+	 * Retrieve values of all variables of a set of devices.
+	 * \param devs Device names
+	 * \return Variable values indexed by variable names, indexed by device names.
+	 */
+	virtual std::map<std::string,std::map<std::string,std::vector<std::string> > > getDevicesVariableValues(const std::set<std::string>& devs);
 	/**
 	 * Intend to set the value of a variable.
 	 * \param dev Device name
 	 * \param name Variable name
 	 * \param value Variable value
-	 */  
-	virtual void setDeviceVariable(const std::string& dev, const std::string& name, const std::string& value)throw(NutException)=0;
+	 */
+	virtual TrackingID setDeviceVariable(const std::string& dev, const std::string& name, const std::string& value) = 0;
 	/**
 	 * Intend to set the value of a variable.
 	 * \param dev Device name
 	 * \param name Variable name
-	 * \param value Variable value
-	 */  
-	virtual void setDeviceVariable(const std::string& dev, const std::string& name, const std::vector<std::string>& values)throw(NutException)=0;
+	 * \param values Vector of variable values
+	 */
+	virtual TrackingID setDeviceVariable(const std::string& dev, const std::string& name, const std::vector<std::string>& values) = 0;
 	/** \} */
 
 	/**
@@ -246,27 +290,28 @@ public:
 	 * \param dev Device name
 	 * \return Command names
 	 */
-	virtual std::set<std::string> getDeviceCommandNames(const std::string& dev)throw(NutException)=0;
+	virtual std::set<std::string> getDeviceCommandNames(const std::string& dev) = 0;
 	/**
 	 * Test if a command is supported by a device.
 	 * \param dev Device name
 	 * \param name Command name
 	 * \return true if the command is supported.
 	 */
-	virtual bool hasDeviceCommand(const std::string& dev, const std::string& name)throw(NutException);
+	virtual bool hasDeviceCommand(const std::string& dev, const std::string& name);
 	/**
 	 * Retrieve the description of a command.
 	 * \param dev Device name
 	 * \param name Command name
 	 * \return Command description if provided.
 	 */
-	virtual std::string getDeviceCommandDescription(const std::string& dev, const std::string& name)throw(NutException)=0;
+	virtual std::string getDeviceCommandDescription(const std::string& dev, const std::string& name) = 0;
 	/**
 	 * Intend to execute a command.
 	 * \param dev Device name
 	 * \param name Command name
+	 * \param param Additional command parameter
 	 */
-	virtual void executeDeviceCommand(const std::string& dev, const std::string& name)throw(NutException)=0;
+	virtual TrackingID executeDeviceCommand(const std::string& dev, const std::string& name, const std::string& param="") = 0;
 	/** \} */
 
 	/**
@@ -277,15 +322,45 @@ public:
 	 * Log the current user (if authenticated) for a device.
 	 * \param dev Device name.
 	 */
-	virtual void deviceLogin(const std::string& dev)throw(NutException)=0;
+	virtual void deviceLogin(const std::string& dev) = 0;
 	/**
-	 * Retrieve the number of user longged in the specified device.
+	 * Retrieve the number of user logged-in for the specified device.
 	 * \param dev Device name.
 	 * \return Number of logged-in users.
 	 */
-	virtual int deviceGetNumLogins(const std::string& dev)throw(NutException)=0;
-	virtual void deviceMaster(const std::string& dev)throw(NutException)=0;
-	virtual void deviceForcedShutdown(const std::string& dev)throw(NutException)=0;
+	virtual int deviceGetNumLogins(const std::string& dev) = 0;
+	/**
+	 * Who did a deviceLogin() to this dev?
+	 * \param dev Device name.
+	 * \return List of clients e.g. {'127.0.0.1', 'admin-workstation.local.domain'}
+	 */
+	virtual std::set<std::string> deviceGetClients(const std::string& dev) = 0;
+	/* NOTE: "master" is deprecated since NUT v2.8.0 in favor of "primary".
+	 * For the sake of old/new server/client interoperability,
+	 * practical implementations should try to use one and fall
+	 * back to the other, and only fail if both return "ERR".
+	 */
+	virtual void deviceMaster(const std::string& dev) = 0;
+	virtual void devicePrimary(const std::string& dev) = 0;
+	virtual void deviceForcedShutdown(const std::string& dev) = 0;
+
+	/**
+	 * Lists all clients of all devices (which have at least one client)
+	 * \return Map with device names vs. list of their connected clients
+	 */
+	virtual std::map<std::string, std::set<std::string>> listDeviceClients(void) = 0;
+
+	/**
+	 * Retrieve the result of a tracking ID.
+	 * \param id Tracking ID.
+	 */
+	virtual TrackingResult getTrackingResult(const TrackingID& id) = 0;
+
+	virtual bool hasFeature(const Feature& feature);
+	virtual bool isFeatureEnabled(const Feature& feature) = 0;
+	virtual void setFeature(const Feature& feature, bool status) = 0;
+
+	static const Feature TRACKING;
 
 protected:
 	Client();
@@ -297,6 +372,11 @@ protected:
  */
 class TcpClient : public Client
 {
+	/* We have a number of direct-call methods we do not expose
+	 * generally, but still want covered with integration tests
+	 */
+	friend class NutActiveClientTest;
+
 public:
 	/**
 	 * Construct a nut TcpClient object.
@@ -309,21 +389,27 @@ public:
 	 * \param host Server host name.
 	 * \param port Server port.
 	 */
-	TcpClient(const std::string& host, int port = 3493)throw(nut::IOException);
-	~TcpClient();
+	TcpClient(const std::string& host, uint16_t port = 3493);
+	~TcpClient() override;
 
 	/**
 	 * Connect it to the specified server.
 	 * \param host Server host name.
 	 * \param port Server port.
 	 */
-	void connect(const std::string& host, int port = 3493)throw(nut::IOException);
+	void connect(const std::string& host, uint16_t port = 3493);
 
 	/**
 	 * Connect to the server.
-	 * Host name and ports must have already set (usefull for reconnection).
+	 * Host name and ports must have already set (useful for reconnection).
 	 */
-	void connect()throw(nut::IOException);
+	void connect();
+
+	/**
+	 * Enable or disable std::cerr tracing of internal Socket operations
+	 * during connect() processing. Primarily for developer troubleshooting.
+	 */
+	void setDebugConnect(bool d);
 
 	/**
 	 * Test if the connection is active.
@@ -339,13 +425,13 @@ public:
 	 * Set the timeout in seconds.
 	 * \param timeout Timeout n seconds, negative to block operations.
 	 */
-	void setTimeout(long timeout);
+	void setTimeout(time_t timeout);
 
 	/**
 	 * Retrieve the timeout.
 	 * \returns Current timeout in seconds.
 	 */
-	long getTimeout()const;
+	time_t getTimeout()const;
 
 	/**
 	 * Retriueve the host name of the server the client is connected to.
@@ -356,53 +442,67 @@ public:
 	 * Retriueve the port of host of the server the client is connected to.
 	 * \return Server port
 	 */
-	int getPort()const;
+	uint16_t getPort()const;
 
-	virtual void authenticate(const std::string& user, const std::string& passwd)throw(NutException);
-	virtual void logout()throw(NutException);
-  
-	virtual Device getDevice(const std::string& name)throw(NutException);
-	virtual std::set<std::string> getDeviceNames()throw(NutException);
-	virtual std::string getDeviceDescription(const std::string& name)throw(NutException);
+	virtual void authenticate(const std::string& user, const std::string& passwd) override;
+	virtual void logout() override;
 
-	virtual std::set<std::string> getDeviceVariableNames(const std::string& dev)throw(NutException);
-	virtual std::set<std::string> getDeviceRWVariableNames(const std::string& dev)throw(NutException);
-	virtual std::string getDeviceVariableDescription(const std::string& dev, const std::string& name)throw(NutException);
-	virtual std::vector<std::string> getDeviceVariableValue(const std::string& dev, const std::string& name)throw(NutException);
-	virtual std::map<std::string,std::vector<std::string> > getDeviceVariableValues(const std::string& dev)throw(NutException);
-	virtual void setDeviceVariable(const std::string& dev, const std::string& name, const std::string& value)throw(NutException);
-	virtual void setDeviceVariable(const std::string& dev, const std::string& name, const std::vector<std::string>& values)throw(NutException);
+	virtual Device getDevice(const std::string& name) override;
+	virtual std::set<std::string> getDeviceNames() override;
+	virtual std::string getDeviceDescription(const std::string& name) override;
 
-	virtual std::set<std::string> getDeviceCommandNames(const std::string& dev)throw(NutException);
-	virtual std::string getDeviceCommandDescription(const std::string& dev, const std::string& name)throw(NutException);
-	virtual void executeDeviceCommand(const std::string& dev, const std::string& name)throw(NutException);
+	virtual std::set<std::string> getDeviceVariableNames(const std::string& dev) override;
+	virtual std::set<std::string> getDeviceRWVariableNames(const std::string& dev) override;
+	virtual std::string getDeviceVariableDescription(const std::string& dev, const std::string& name) override;
+	virtual std::vector<std::string> getDeviceVariableValue(const std::string& dev, const std::string& name) override;
+	virtual std::map<std::string,std::vector<std::string> > getDeviceVariableValues(const std::string& dev) override;
+	virtual std::map<std::string,std::map<std::string,std::vector<std::string> > > getDevicesVariableValues(const std::set<std::string>& devs) override;
+	virtual TrackingID setDeviceVariable(const std::string& dev, const std::string& name, const std::string& value) override;
+	virtual TrackingID setDeviceVariable(const std::string& dev, const std::string& name, const std::vector<std::string>& values) override;
 
- 	virtual void deviceLogin(const std::string& dev)throw(NutException);
-	virtual void deviceMaster(const std::string& dev)throw(NutException);
-	virtual void deviceForcedShutdown(const std::string& dev)throw(NutException);
-	virtual int deviceGetNumLogins(const std::string& dev)throw(NutException);
+	virtual std::set<std::string> getDeviceCommandNames(const std::string& dev) override;
+	virtual std::string getDeviceCommandDescription(const std::string& dev, const std::string& name) override;
+	virtual TrackingID executeDeviceCommand(const std::string& dev, const std::string& name, const std::string& param="") override;
+
+	virtual void deviceLogin(const std::string& dev) override;
+	/* FIXME: Protocol update needed to handle master/primary alias
+	 * and probably an API bump also, to rename/alias the routine.
+	 */
+	virtual void deviceMaster(const std::string& dev) override;
+	virtual void devicePrimary(const std::string& dev) override;
+	virtual void deviceForcedShutdown(const std::string& dev) override;
+	virtual int deviceGetNumLogins(const std::string& dev) override;
+	virtual std::set<std::string> deviceGetClients(const std::string& dev) override;
+
+	virtual std::map<std::string, std::set<std::string>> listDeviceClients(void) override;
+
+	virtual TrackingResult getTrackingResult(const TrackingID& id) override;
+
+	virtual bool isFeatureEnabled(const Feature& feature) override;
+	virtual void setFeature(const Feature& feature, bool status) override;
 
 protected:
-	std::string sendQuery(const std::string& req)throw(nut::IOException);
-	static void detectError(const std::string& req)throw(nut::NutException);
+	std::string sendQuery(const std::string& req);
+	void sendAsyncQueries(const std::vector<std::string>& req);
+	static void detectError(const std::string& req);
+	TrackingID sendTrackingQuery(const std::string& req);
 
-	std::vector<std::string> get(const std::string& subcmd, const std::string& params = "")
-		throw(nut::NutException);
+	std::vector<std::string> get(const std::string& subcmd, const std::string& params = "");
 
-	std::vector<std::vector<std::string> > list(const std::string& subcmd, const std::string& params = "")
-		throw(nut::NutException);
+	std::vector<std::vector<std::string> > list(const std::string& subcmd, const std::string& params = "");
+
+	std::vector<std::vector<std::string> > parseList(const std::string& req);
 
 	static std::vector<std::string> explode(const std::string& str, size_t begin=0);
 	static std::string escape(const std::string& str);
 
 private:
 	std::string _host;
-	int _port;
-	long _timeout;
+	uint16_t _port;
+	time_t _timeout;
 	internal::Socket* _socket;
 };
 
-
 /**
  * Device attached to a client.
  * Device is a lightweight class which can be copied easily.
@@ -411,9 +511,14 @@ class Device
 {
 	friend class Client;
 	friend class TcpClient;
+	friend class TcpClientMock;
+#ifdef _NUTCLIENTTEST_BUILD
+	friend class NutClientTest;
+#endif
 public:
 	~Device();
 	Device(const Device& dev);
+	Device& operator=(const Device& dev);
 
 	/**
 	 * Retrieve the name of the device.
@@ -435,112 +540,121 @@ public:
 	bool isOk()const;
 	/**
 	 * Test if the device is valid (has a name and is attached to a client).
-     * @see Device::isOk()
+	 * @see Device::isOk()
 	 */
 	operator bool()const;
 	/**
 	 * Test if the device is not valid (has no name or is not attached to any client).
-     * @see Device::isOk()
+	 * @see Device::isOk()
 	 */
 	bool operator!()const;
 	/**
 	 * Test if the two devices are sames (same name ad same client attached to).
 	 */
 	bool operator==(const Device& dev)const;
-  /**
-   * Comparison operator.
-   */
-  bool operator<(const Device& dev)const;
+	/**
+	 * Comparison operator.
+	 */
+	bool operator<(const Device& dev)const;
 
 	/**
 	 * Retrieve the description of the devce if specified.
 	 */
-	std::string getDescription()throw(NutException);
+	std::string getDescription();
 
 	/**
 	 * Intend to retrieve the value of a variable of the device.
 	 * \param name Name of the variable to get.
-     * \return Value of the variable, if available.
+	 * \return Value of the variable, if available.
 	 */
-	std::vector<std::string> getVariableValue(const std::string& name)throw(NutException);
+	std::vector<std::string> getVariableValue(const std::string& name);
 	/**
 	 * Intend to retrieve values of all variables of the devices.
 	 * \return Map of all variables values indexed by their names.
 	 */
-	std::map<std::string,std::vector<std::string> > getVariableValues()throw(NutException);
+	std::map<std::string,std::vector<std::string> > getVariableValues();
 	/**
 	 * Retrieve all variables names supported by the device.
 	 * \return Set of available variable names.
 	 */
-	std::set<std::string> getVariableNames()throw(NutException);
+	std::set<std::string> getVariableNames();
 	/**
 	 * Retrieve all Read/Write variables names supported by the device.
 	 * \return Set of available Read/Write variable names.
 	 */
-	std::set<std::string> getRWVariableNames()throw(NutException);
+	std::set<std::string> getRWVariableNames();
 	/**
 	 * Intend to set the value of a variable of the device.
 	 * \param name Variable name.
 	 * \param value New variable value.
 	 */
-	void setVariable(const std::string& name, const std::string& value)throw(NutException);
+	void setVariable(const std::string& name, const std::string& value);
 	/**
 	 * Intend to set values of a variable of the device.
 	 * \param name Variable name.
-	 * \param value New variable values.
+	 * \param values Vector of new variable values.
 	 */
-	void setVariable(const std::string& name, const std::vector<std::string>& values)throw(NutException);
+	void setVariable(const std::string& name, const std::vector<std::string>& values);
 
 	/**
 	 * Retrieve a Variable object representing the specified variable.
-     * \param name Variable name.
+	 * \param name Variable name.
 	 * \return Variable object.
 	 */
-	Variable getVariable(const std::string& name)throw(NutException);
+	Variable getVariable(const std::string& name);
 	/**
 	 * Retrieve Variable objects representing all variables available for the device.
 	 * \return Set of Variable objects.
 	 */
-	std::set<Variable> getVariables()throw(NutException);
+	std::set<Variable> getVariables();
 	/**
 	 * Retrieve Variable objects representing all Read/Write variables available for the device.
 	 * \return Set of Variable objects.
 	 */
-	std::set<Variable> getRWVariables()throw(NutException);
+	std::set<Variable> getRWVariables();
 
 	/**
 	 * Retrieve names of all commands supported by the device.
 	 * \return Set of available command names.
 	 */
-	std::set<std::string> getCommandNames()throw(NutException);
+	std::set<std::string> getCommandNames();
 	/**
 	 * Retrieve objects for all commands supported by the device.
 	 * \return Set of available Command objects.
 	 */
-	std::set<Command> getCommands()throw(NutException);
+	std::set<Command> getCommands();
 	/**
 	 * Retrieve an object representing a command of the device.
 	 * \param name Command name.
 	 * \return Command object.
 	 */
-	Command getCommand(const std::string& name)throw(NutException);
+	Command getCommand(const std::string& name);
 	/**
 	 * Intend to execute a command on the device.
 	 * \param name Command name.
+	 * \param param Additional command parameter
 	 */
-	void executeCommand(const std::string& name)throw(NutException);
+	TrackingID executeCommand(const std::string& name, const std::string& param="");
 
 	/**
 	 * Login current client's user for the device.
 	 */
-	void login()throw(NutException);
-	void master()throw(NutException);
-	void forcedShutdown()throw(NutException);
+	void login();
+	/**
+	 * Who did a login() to this dev?
+	 */
+	std::set<std::string> getClients();
+	/* FIXME: Protocol update needed to handle master/primary alias
+	 * and probably an API bump also, to rename/alias the routine.
+	 */
+	void master();
+	void primary();
+	void forcedShutdown();
 	/**
 	 * Retrieve the number of logged user for the device.
 	 * \return Number of users.
 	 */
-	int getNumLogins()throw(NutException);
+	int getNumLogins();
 
 protected:
 	Device(Client* client, const std::string& name);
@@ -558,10 +672,15 @@ class Variable
 {
 	friend class Device;
 	friend class TcpClient;
+	friend class TcpClientMock;
+#ifdef _NUTCLIENTTEST_BUILD
+	friend class NutClientTest;
+#endif
 public:
 	~Variable();
 
 	Variable(const Variable& var);
+	Variable& operator=(const Variable& var);
 
 	/**
 	 * Retrieve variable name.
@@ -603,23 +722,23 @@ public:
 	 * Intend to retrieve variable value.
 	 * \return Value of the variable.
 	 */
-	std::vector<std::string> getValue()throw(NutException);
+	std::vector<std::string> getValue();
 	/**
 	 * Intend to retireve variable description.
 	 * \return Variable description if provided.
 	 */
-	std::string getDescription()throw(NutException);
+	std::string getDescription();
 
 	/**
 	 * Intend to set a value to the variable.
 	 * \param value New variable value.
 	 */
-	void setValue(const std::string& value)throw(NutException);
+	void setValue(const std::string& value);
 	/**
 	 * Intend to set (multiple) values to the variable.
-	 * \param value New variable values.
+	 * \param values Vector of new variable values.
 	 */
-	void setValues(const std::vector<std::string>& values)throw(NutException);
+	void setValues(const std::vector<std::string>& values);
 
 protected:
 	Variable(Device* dev, const std::string& name);
@@ -637,10 +756,15 @@ class Command
 {
 	friend class Device;
 	friend class TcpClient;
+	friend class TcpClientMock;
+#ifdef _NUTCLIENTTEST_BUILD
+	friend class NutClientTest;
+#endif
 public:
 	~Command();
 
 	Command(const Command& cmd);
+	Command& operator=(const Command& cmd);
 
 	/**
 	 * Retrieve command name.
@@ -683,13 +807,13 @@ public:
 	 * Intend to retireve command description.
 	 * \return Command description if provided.
 	 */
-	std::string getDescription()throw(NutException);
+	std::string getDescription();
 
 	/**
-	 * Intend to retrieve command description.
-	 * \return Command description if provided.
+	 * Intend to execute the instant command on device.
+	 * \param param Optional additional command parameter
 	 */
-	void execute()throw(NutException);
+	void execute(const std::string& param="");
 
 protected:
 	Command(Device* dev, const std::string& name);
@@ -722,13 +846,19 @@ typedef char** strarr;
 /**
  * Alloc an array of string.
  */
-strarr strarr_alloc(unsigned short count);
+strarr strarr_alloc(size_t count);
 
 /**
  * Free an array of string.
  */
 void strarr_free(strarr arr);
 
+/**
+ * Convert C++ types into an array of string.
+ */
+strarr stringvector_to_strarr(const std::vector<std::string>& strset);
+strarr stringset_to_strarr(const std::set<std::string>& strset);
+
 
 /**
  * Nut general client types and functions.
@@ -738,7 +868,7 @@ void strarr_free(strarr arr);
 typedef void* NUTCLIENT_t;
 
 /**
- * Destroy a client. 
+ * Destroy a client.
  * \param client Nut client handle.
  */
 void nutclient_destroy(NUTCLIENT_t client);
@@ -776,7 +906,11 @@ int nutclient_get_device_num_logins(NUTCLIENT_t client, const char* dev);
  * \param client Nut client handle.
  * \param dev Device name to test.
  */
+/* FIXME: Protocol update needed to handle master/primary alias
+ * and probably an API bump also, to rename/alias the routine.
+ */
 void nutclient_device_master(NUTCLIENT_t client, const char* dev);
+void nutclient_device_primary(NUTCLIENT_t client, const char* dev);
 
 /**
  * Set the FSD flag for the device.
@@ -901,7 +1035,7 @@ char* nutclient_get_device_command_description(NUTCLIENT_t client, const char* d
  * \param dev Device name.
  * \param cmd Command name.
  */
-void nutclient_execute_device_command(NUTCLIENT_t client, const char* dev, const char* cmd);
+void nutclient_execute_device_command(NUTCLIENT_t client, const char* dev, const char* cmd, const char* param="");
 
 /** \} */
 
@@ -911,7 +1045,7 @@ void nutclient_execute_device_command(NUTCLIENT_t client, const char* dev, const
  * \{
  */
 /**
- * Hidden structure representing a TCP connection to NUTD. 
+ * Hidden structure representing a TCP connection to NUTD.
  * NUTCLIENT_TCP_t is back compatible to NUTCLIENT_t.
  */
 typedef NUTCLIENT_t NUTCLIENT_TCP_t;
@@ -920,9 +1054,9 @@ typedef NUTCLIENT_t NUTCLIENT_TCP_t;
  * Create a client to NUTD using a TCP connection.
  * \param host Host name to connect to.
  * \param port Host port.
- * \return New client or NULL if failed.
+ * \return New client or nullptr if failed.
  */
-NUTCLIENT_TCP_t nutclient_tcp_create_client(const char* host, unsigned short port);
+NUTCLIENT_TCP_t nutclient_tcp_create_client(const char* host, uint16_t port);
 /**
  * Test if a nut TCP client is connected.
  * \param client Nut TCP client handle.
@@ -945,12 +1079,12 @@ int nutclient_tcp_reconnect(NUTCLIENT_TCP_t client);
  * Set the timeout value for the TCP connection.
  * \param timeout Timeout in seconds, negative for blocking.
  */
-void nutclient_tcp_set_timeout(NUTCLIENT_TCP_t client, long timeout);
+void nutclient_tcp_set_timeout(NUTCLIENT_TCP_t client, time_t timeout);
 /**
  * Retrieve the timeout value for the TCP connection.
  * \return Timeout value in seconds.
  */
-long nutclient_tcp_get_timeout(NUTCLIENT_TCP_t client);
+time_t nutclient_tcp_get_timeout(NUTCLIENT_TCP_t client);
 
 /** \} */
 
